<?php

namespace Drupal\ohdear_integration\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Cache\CacheableJsonResponse;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\ohdear_integration\OhDearHealthcheckGenerator;
use Drupal\ohdear_integration\OhDearSdkService;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Returns responses for OhDear Integration routes.
 */
class OhDearIntegrationController extends ControllerBase {

  /**
   * Healthcheck generator service.
   *
   * @var \Drupal\ohdear_integration\OhDearHealthcheckGenerator
   */
  protected $ohDearGenerator;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The current request.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The core renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The OhDear service development kit and helper service.
   *
   * @var \Drupal\ohdear_integration\OhDearSdkService
   */
  protected $ohDearSdk;

  /**
   * Constructs OhDear integration controller.
   *
   * @param \Drupal\ohdear_integration\OhDearHealthcheckGenerator $ohdear_generator
   *   The ohdear helper objects factory service.
   * @param \Psr\Log\LoggerInterface $logger
   *   The simple sitemap extensions logger service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The core renderer service.
   */
  public function __construct(OhDearHealthcheckGenerator $ohdear_generator, LoggerInterface $logger, RequestStack $request_stack, RendererInterface $renderer, OhDearSdkService $ohdear_sdk) {
    $this->ohDearGenerator = $ohdear_generator;
    $this->logger = $logger;
    $this->requestStack = $request_stack;
    $this->renderer = $renderer;
    $this->ohDearSdk = $ohdear_sdk;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): OhDearIntegrationController {
    return new static(
      $container->get('ohdear_healthcheck.generator'),
      $container->get('logger.channel.ohdear_integration'),
      $container->get('request_stack'),
      $container->get('renderer'),
      $container->get('ohdear_sdk')
    );
  }

  /**
   * Builds the response.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Response.
   */
  public function buildJson() {
    // Render response in context to avoid metadata leaks. Some monitoring
    // plugins are rendered on sensor run and cached at that time as well. That
    // metadata is hard to access, so we need rendering in context detour.
    $context = new RenderContext();
    $response = $this->renderer->executeInRenderContext($context, function () {
      if ($this->access($this->currentUser())->isForbidden()) {
        $data = '{"error": "Access denied!"}';
        $json_response = new CacheableJsonResponse(
          $data,
          403,
          ['Cache-Control' => 'no-cache'],
          TRUE
        );
      }
      else {
        $data = $this->ohDearGenerator->getData();
        $json_response = new CacheableJsonResponse(
          $data,
          200,
          ['Cache-Control' => 'no-cache'],
          TRUE
        );
      }
      $cacheable_metadata = $json_response->getCacheableMetadata();
      $this->buildCache($cacheable_metadata);
      $json_response->addCacheableDependency($cacheable_metadata);
      return $json_response;
    });
    // If there is metadata left on the context, apply it on the response.
    if (!$context->isEmpty()) {
      if ($metadata = $context->pop()) {
        $this->buildCache($metadata);
        $response->addCacheableDependency($metadata);
      }
    }
    return $response;
  }

  /**
   * Access result callback.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   Determines the access to controller.
   */
  public function access(AccountInterface $account) {
    $ohdear_header_secret = $this->requestStack->getCurrentRequest()->headers->get('oh-dear-health-check-secret') ?? NULL;
    $ohdear_healthcheck_secret = $this->ohDearSdk->getHealthcheckSecret();
    if (($ohdear_header_secret && $ohdear_header_secret === $ohdear_healthcheck_secret)
      || $account->hasPermission('monitoring reports')) {
      $access_result = AccessResult::allowed();
    }
    else {
      $access_result = AccessResult::forbidden();
      $this->logger->warning('Unauthorized access to monitoring results denied.');
    }
    $access_result
      ->setCacheMaxAge(0)
      ->addCacheContexts([
        'headers:oh-dear-health-check-secret',
        'user.roles',
      ])
      ->addCacheTags(['monitoring_sensor_result']);
    return $access_result;
  }

  /**
   * Make sure ohdear healthcheck response is not cached.
   *
   * @param \Drupal\Core\Cache\CacheableMetadata $cacheable_metadata
   *   Cacheable metadata.
   */
  protected function buildCache(CacheableMetadata $cacheable_metadata) {
    $cacheable_metadata->setCacheContexts([
      'headers:oh-dear-health-check-secret',
      'user.roles',
    ]);
    $cacheable_metadata->setCacheTags(['monitoring_sensor_result']);
    $cacheable_metadata->setCacheMaxAge(0);
  }

}
